@@ -34,6 +34,10 @@
 
 #include "basic-common.h"
 
+/* No extra fields needed */
+typedef PangoEngineShape      BasicEngineWin32;
+typedef PangoEngineShapeClass BasicEngineWin32Class ;
+
 #define SCRIPT_ENGINE_NAME "BasicScriptEngineWin32"
 
 static gboolean pango_win32_debug = FALSE;
@@ -122,89 +126,47 @@ static int nscripts;
 #endif
 
 #ifdef HAVE_USP10_H
-static PangoEngineRange uniscribe_ranges[] = {
+static PangoEngineScriptInfo uniscribe_scripts[] = {
   /* We claim to cover everything ;-) */
-  { 0x0020, 0xffee, "*" }
+  { PANGO_SCRIPT_COMMON,  "" },
 };
 #endif
 
-static PangoEngineRange basic_ranges[] = {
+static PangoEngineScriptInfo basic_scripts[] = {
   /* Those characters that can be rendered legibly without Uniscribe.
    * I am not certain this list is correct.
    */
+  { PANGO_SCRIPT_ARMENIAN, "*" },
+  { PANGO_SCRIPT_BOPOMOFO, "*" },
+  { PANGO_SCRIPT_CHEROKEE, "*" },
+  { PANGO_SCRIPT_COPTIC,   "*" },
+  { PANGO_SCRIPT_CYRILLIC, "*" },
+  { PANGO_SCRIPT_DESERET,  "*" },
+  { PANGO_SCRIPT_ETHIOPIC, "*" },
+  { PANGO_SCRIPT_GEORGIAN, "*" },
+  { PANGO_SCRIPT_GOTHIC,   "*" },
+  { PANGO_SCRIPT_GREEK,    "*" },
+  { PANGO_SCRIPT_HAN,      "*" },
+  { PANGO_SCRIPT_HANGUL,   "*" },
+  { PANGO_SCRIPT_HIRAGANA, "*" },
+  { PANGO_SCRIPT_KATAKANA, "*" },
+  { PANGO_SCRIPT_LATIN,    "*" },
+  { PANGO_SCRIPT_OGHAM,    "*" },
+  { PANGO_SCRIPT_OLD_ITALIC, "*" },
+  { PANGO_SCRIPT_RUNIC,     "*" },
+  { PANGO_SCRIPT_THAI,      "*" },
+  { PANGO_SCRIPT_CANADIAN_ABORIGINAL, "*" },
+  { PANGO_SCRIPT_YI,       "*" },
+  { PANGO_SCRIPT_BRAILLE,  "*" },
+  { PANGO_SCRIPT_CYPRIOT,  "*" },
+  { PANGO_SCRIPT_LIMBU,    "*" },
+  { PANGO_SCRIPT_OSMANYA,  "*" },
+  { PANGO_SCRIPT_SHAVIAN,  "*" },
+  { PANGO_SCRIPT_LINEAR_B, "*" },
+  { PANGO_SCRIPT_UGARITIC, "*" },
 
-  /* Basic Latin, Latin-1 Supplement, Latin Extended-A, Latin Extended-B,
-   * IPA Extensions
-   */
-  { 0x0020, 0x02af, "*" },
-
-  /* Spacing Modifier Letters */
-  { 0x02b0, 0x02ff, "" },
-
-  /* Not covered: Combining Diacritical Marks */
-
-  /* Greek, Cyrillic, Armenian */
-  { 0x0380, 0x058f, "*" },
-
-  /* Hebrew */
-  { 0x0591, 0x05f4, "*" },
-
-  /* Arabic */
-  { 0x060c, 0x06f9, "" },
-
-  /* Not covered: Syriac, Thaana, Devanagari, Bengali, Gurmukhi, Gujarati,
-   * Oriya, Tamil, Telugu, Kannada, Malayalam, Sinhala
-   */
-
-  /* Thai */
-  { 0x0e01, 0x0e5b, "" },
-
-  /* Not covered: Lao, Tibetan, Myanmar */
-
-  /* Georgian */
-  { 0x10a0, 0x10ff, "*" },
-
-  /* Not covered: Hangul Jamo */
-
-  /* Ethiopic, Cherokee, Unified Canadian Aboriginal Syllabics, Ogham,
-   * Runic */
-  { 0x1200, 0x16ff, "*" },
-
-  /* Not covered: Khmer, Mongolian */
-
-  /* Latin Extended Additional, Greek Extended */
-  { 0x1e00, 0x1fff, "*" },
-
-  /* General Punctuation, Superscripts and Subscripts, Currency
-   * Symbols, Combining Marks for Symbols, Letterlike Symbols, Number
-   * Forms, Arrows, Mathematical Operators, Miscellaneous Technical,
-   * Control Pictures, Optical Character Recognition, Enclosed
-   * Alphanumerics, Box Drawing, Block Elements, Geometric Shapes,
-   * Miscellaneous Symbols, Dingbats, Braille Patterns, CJK Radicals
-   * Supplement, Kangxi Radicals, Ideographic Description Characters,
-   * CJK Symbols and Punctuation, Hiragana, Katakana, Bopomofo, Hangul
-   * Compatibility Jamo, Kanbun, Bopomofo Extended, Enclosed CJK
-   * Letters and Months, CJK Compatibility, CJK Unified Ideographs
-   * Extension A, CJK Unified Ideographs, Yi Syllables, Yi Radicals
-   */
-  { 0x2000, 0xa4c6, "*" },
-
-  /* Hangul Syllables */
-  { 0xac00, 0xd7a3, "kr" },
-
-  /* Not covered: Private Use */
-
-  /* CJK Compatibility Ideographs (partly) */
-  { 0xf900, 0xfa0b, "kr" },
-
-  /* Not covered: CJK Compatibility Ideographs (partly), Alphabetic
-   * Presentation Forms, Arabic Presentation Forms-A, Combining Half
-   * Marks, CJK Compatibility Forms, Small Form Variants, Arabic
-   * Presentation Forms-B, Specials
-   */
-
-  /* Halfwidth and Fullwidth Forms */
-  { 0xff00, 0xffed, "*" }
+  /* Claim to handle everything as a fallback */
+  { PANGO_SCRIPT_COMMON,   "" }
 };
 
 static PangoEngineInfo script_engines[] = {
@@ -314,23 +276,35 @@ lang_name (int lang)
     {
 #define CASE(n) case LANG_##n: return #n
       CASE (NEUTRAL);
+#ifdef LANG_INVARIANT
       CASE (INVARIANT);
+#endif
       CASE (AFRIKAANS);
       CASE (ALBANIAN);
       CASE (ARABIC);
+#ifdef LANG_ARMENIAN
       CASE (ARMENIAN);
+#endif
+#ifdef LANG_ASSAMESE
       CASE (ASSAMESE);
+#endif
+#ifdef LANG_AZERI
       CASE (AZERI);
+#endif
       CASE (BASQUE);
       CASE (BELARUSIAN);
+#ifdef LANG_BENGALI
       CASE (BENGALI);
+#endif
       CASE (BULGARIAN);
       CASE (CATALAN);
       CASE (CHINESE);
       CASE (CROATIAN);
       CASE (CZECH);
       CASE (DANISH);
+#ifdef LANG_DIVEHI
       CASE (DIVEHI);
+#endif
       CASE (DUTCH);
       CASE (ENGLISH);
       CASE (ESTONIAN);
@@ -338,56 +312,110 @@ lang_name (int lang)
       CASE (FARSI);
       CASE (FINNISH);
       CASE (FRENCH);
+#ifdef LANG_GALICIAN
       CASE (GALICIAN);
+#endif
+#ifdef LANG_GEORGIAN
       CASE (GEORGIAN);
+#endif
       CASE (GERMAN);
       CASE (GREEK);
+#ifdef LANG_GUJARATI
       CASE (GUJARATI);
+#endif
       CASE (HEBREW);
+#ifdef LANG_HINDI
       CASE (HINDI);
+#endif
       CASE (HUNGARIAN);
       CASE (ICELANDIC);
       CASE (INDONESIAN);
       CASE (ITALIAN);
       CASE (JAPANESE);
+#ifdef LANG_KANNADA
       CASE (KANNADA);
+#endif
+#ifdef LANG_KASHMIRI
       CASE (KASHMIRI);
+#endif
+#ifdef LANG_KAZAK
       CASE (KAZAK);
+#endif
+#ifdef LANG_KONKANI
       CASE (KONKANI);
+#endif
       CASE (KOREAN);
+#ifdef LANG_KYRGYZ
       CASE (KYRGYZ);
+#endif
       CASE (LATVIAN);
       CASE (LITHUANIAN);
+#ifdef LANG_MACEDONIAN
       CASE (MACEDONIAN);
+#endif
+#ifdef LANG_MALAY
       CASE (MALAY);
+#endif
+#ifdef LANG_MALAYALAM
       CASE (MALAYALAM);
+#endif
+#ifdef LANG_MANIPURI
       CASE (MANIPURI);
+#endif
+#ifdef LANG_MARATHI
       CASE (MARATHI);
+#endif
+#ifdef LANG_MONGOLIAN
       CASE (MONGOLIAN);
+#endif
+#ifdef LANG_NEPALI
       CASE (NEPALI);
+#endif
       CASE (NORWEGIAN);
+#ifdef LANG_ORIYA
       CASE (ORIYA);
+#endif
       CASE (POLISH);
       CASE (PORTUGUESE);
+#ifdef LANG_PUNJABI
       CASE (PUNJABI);
+#endif
       CASE (ROMANIAN);
       CASE (RUSSIAN);
+#ifdef LANG_SANSKRIT
       CASE (SANSKRIT);
+#endif
+#ifdef LANG_SINDHI
       CASE (SINDHI);
+#endif
       CASE (SLOVAK);
       CASE (SLOVENIAN);
       CASE (SPANISH);
+#ifdef LANG_SWAHILI
       CASE (SWAHILI);
+#endif
       CASE (SWEDISH);
+#ifdef LANG_SYRIAC
       CASE (SYRIAC);
+#endif
+#ifdef LANG_TAMIL
       CASE (TAMIL);
+#endif
+#ifdef LANG_TATAR
       CASE (TATAR);
+#endif
+#ifdef LANG_TELUGU
       CASE (TELUGU);
+#endif
       CASE (THAI);
       CASE (TURKISH);
       CASE (UKRAINIAN);
+#ifdef LANG_URDU
       CASE (URDU);
+#endif
+#ifdef LANG_UZBEK
       CASE (UZBEK);
+#endif
       CASE (VIETNAMESE);
 #undef CASE
     default:
@@ -944,7 +972,8 @@ text_is_simple (const char *text,
 #endif /* HAVE_USP10_H */
 
 static void 
-basic_engine_shape (PangoFont        *font,
+basic_engine_shape (PangoEngineShape *engine,
+		    PangoFont        *font,
 		    const char       *text,
 		    gint              length,
 		    PangoAnalysis    *analysis,
@@ -1050,29 +1079,6 @@ basic_engine_shape (PangoFont        *font,
     }
 }
 
-static PangoCoverage *
-basic_engine_get_coverage (PangoFont     *font,
-			   PangoLanguage *lang)
-{
-  return pango_font_get_coverage (font, lang);
-}
-
-static PangoEngine *
-basic_engine_win32_new (void)
-{
-  PangoEngineShape *result;
-  
-  result = g_new (PangoEngineShape, 1);
-
-  result->engine.id = SCRIPT_ENGINE_NAME;
-  result->engine.type = PANGO_ENGINE_TYPE_SHAPE;
-  result->engine.length = sizeof (result);
-  result->script_shape = basic_engine_shape;
-  result->get_coverage = basic_engine_get_coverage;
-
-  return (PangoEngine *)result;
-}
-
 static void
 init_uniscribe (void)
 {
@@ -1110,23 +1116,39 @@ init_uniscribe (void)
 #endif
 } 
 
-/* The following three functions provide the public module API for
- * Pango
- */
-#ifdef WIN32_MODULE_PREFIX
-#define MODULE_ENTRY(func) _pango_basic_win32_##func
-#else
-#define MODULE_ENTRY(func) func
-#endif
+static void
+basic_engine_win32_class_init (PangoEngineShapeClass *class)
+{
+  class->script_shape = basic_engine_shape;
+}
 
-void
-MODULE_ENTRY(script_engine_list) (PangoEngineInfo **engines,
-				  gint             *n_engines)
+PANGO_ENGINE_SHAPE_DEFINE_TYPE (BasicEngineWin32, basic_engine_win32,
+				basic_engine_win32_class_init, NULL);
+
+void 
+PANGO_MODULE_ENTRY(init) (GTypeModule *module)
 {
   init_uniscribe ();
 
-  script_engines[0].ranges = basic_ranges;
-  script_engines[0].n_ranges = G_N_ELEMENTS (basic_ranges);
+  if (pango_win32_get_debug_flag ())
+    pango_win32_debug = TRUE;
+
+  basic_engine_register_type (module);
+}
+
+void 
+PANGO_MODULE_ENTRY(exit) (void)
+{
+}
+
+void 
+PANGO_MODULE_ENTRY(list) (PangoEngineInfo **engines,
+			  int              *n_engines)
+{
+  init_uniscribe ();
+
+  script_engines[0].scripts = basic_scripts;
+  script_engines[0].n_scripts = G_N_ELEMENTS (basic_scripts);
 
 #ifdef HAVE_USP10_H
   if (have_uniscribe)
@@ -1158,20 +1180,10 @@ MODULE_ENTRY(script_engine_list) (PangoEngineInfo **engines,
 }
 
 PangoEngine *
-MODULE_ENTRY(script_engine_load) (const char *id)
+PANGO_MODULE_ENTRY(create) (const char *id)
 {
-  init_uniscribe ();
-
-  if (pango_win32_get_debug_flag ())
-    pango_win32_debug = TRUE;
-
   if (!strcmp (id, SCRIPT_ENGINE_NAME))
-    return basic_engine_win32_new ();
+    return g_object_new (basic_engine_win32_type, NULL);
   else
     return NULL;
 }
-
-void 
-MODULE_ENTRY(script_engine_unload) (PangoEngine *engine)
-{
-}
