@@ -24,15 +24,15 @@
 #include <math.h>
 
 #include <X11/Xlib.h>
-#include "pangox.h"
 #include "pango-utils.h"
-#include "pangox-private.h"
-#include "pango-intset.h"
 #include "modules.h"
 
-#define PANGO_X_UNKNOWN_FLAG 0x10000000
+#undef PANGO_DISABLE_DEPRECATED
 
-#define PANGO_LIGATURE_HACK_DEBUG
+#include "pangox.h"
+#include "pangox-private.h"
+
+#define PANGO_X_UNKNOWN_FLAG 0x10000000
 
 #include "config.h"
 
@@ -46,97 +46,6 @@
 typedef struct _PangoXFontClass   PangoXFontClass;
 typedef struct _PangoXMetricsInfo PangoXMetricsInfo;
 typedef struct _PangoXContextInfo PangoXContextInfo;
-typedef struct _PangoXLigatureInfo   PangoXLigatureInfo;
-typedef struct _PangoXLigatureSource PangoXLigatureSource;
-
-#ifndef HAVE_STRTOK_R
-/* This implementation of strtok_r comes from the GNU C library.
- * Copyright (C) 1991, 1996, 1997, 1998, 1999, 2001 Free Software Foundation, Inc.
- */
-static char *
-my_strtok_r (char       *s,
-	     const char *delim,
-	     char      **save_ptr)
-{
-  char *token;
-
-  if (s == NULL)
-    s = *save_ptr;
-
-  /* Scan leading delimiters.  */
-  s += strspn (s, delim);
-  if (*s == '\0')
-    {
-      *save_ptr = s;
-      return NULL;
-    }
-
-  /* Find the end of the token.  */
-  token = s;
-  s = strpbrk (token, delim);
-  if (s == NULL)
-    /* This token finishes the string.  */
-    *save_ptr = token + strlen (token);
-  else
-    {
-      /* Terminate the token and make *SAVE_PTR point past it.  */
-      *s = '\0';
-      *save_ptr = s + 1;
-    }
-  return token;
-}
-#else
-#define my_strtok_r strtok_r
-#endif /* HAVE_STRTOK_R */
-
-static int
-hex_to_integer (const char *s) 
-{
-  int a;
-  char *end_ptr;
-  
-  if (!*s)
-    return (gunichar)-1;
-
-  a = strtol (s, &end_ptr, 16);
-  if (*end_ptr)
-    return (gunichar)-1;  /* Invalid characters in string */
-    
-  if (a <= 0 || a >= 0xffff)
-    return (gunichar)-1; /* Character out of range */
-  
-  return a;
-}
-
-static PangoIntSet *
-parse_gintset_spec (char *s) 
-{
-  char *m = NULL;
-  PangoIntSet *set = pango_int_set_new ();
-  s = my_strtok_r (s, ",", &m);
-  while (s)
-    {
-      char *p = strchr (s, '-');
-      if (!p) 
-	{
-	  int i = hex_to_integer (s);
-	  if (i != -1)
-	    pango_int_set_add (set, i);
-	}
-      else 
-	{
-	  int start, end;
-	  *p = 0;
-	  p++;
-	  start = hex_to_integer (s);
-	  end = hex_to_integer (p);
-	  if (start != -1 && end != -1)
-	    pango_int_set_add_range (set, start, end);
-	}
-      s = my_strtok_r (NULL, ",", &m);
-    }
-  return set;
-}
 
 struct _PangoXSubfontInfo
 {
@@ -145,12 +54,6 @@ struct _PangoXSubfontInfo
   gboolean     is_1byte;
   int          range_byte1;
   int          range_byte2;
-  
-  /* hash table mapping setnames to PangoIntSets */
-  GHashTable *ligature_sets;
-   
-  PangoXLigatureInfo *ligs;
-  int n_ligs;
 };
 
 struct _PangoXMetricsInfo
@@ -170,24 +73,6 @@ struct _PangoXFontClass
   PangoFontClass parent_class;
 };
 
-struct _PangoXLigatureSource
-{
-  gboolean is_set;
-  union {
-    PangoIntSet *set;
-    gunichar glyph;
-  } data;
-};
-
-struct _PangoXLigatureInfo 
-{
-  int n_source;
-  PangoXLigatureSource *source;
-
-  gunichar *dest;
-  int n_dest;
-};
-
 static PangoFontClass *parent_class;	/* Parent class structure for PangoXFont */
 
 static void pango_x_font_class_init (PangoXFontClass *class);
@@ -227,11 +112,6 @@ static void     pango_x_get_item_properties (PangoItem      *item,
 					     PangoAttrColor *bg_color,
 					     gboolean       *bg_set);
 
-static void font_struct_get_ligatures (PangoFontMap *map,
-				       Display *display, 
-				       XFontStruct *fs, 
-				       PangoXSubfontInfo *info);
-
 static inline PangoXSubfontInfo *
 pango_x_find_subfont (PangoFont  *font,
 		      PangoXSubfont subfont_index)
@@ -267,7 +147,6 @@ pango_x_make_font_struct (PangoFont *font, PangoXSubfontInfo *info)
   info->is_1byte = (info->font_struct->min_byte1 == 0 && info->font_struct->max_byte1 == 0);
   info->range_byte1 = info->font_struct->max_byte1 - info->font_struct->min_byte1 + 1;
   info->range_byte2 = info->font_struct->max_char_or_byte2 - info->font_struct->min_char_or_byte2 + 1;
-  font_struct_get_ligatures (xfont->fontmap, xfont->display, info->font_struct, info);
 }
 
 static inline XFontStruct *
@@ -414,10 +293,10 @@ pango_x_font_new (PangoFontMap *fontmap, const char *spec, int size)
   g_return_val_if_fail (fontmap != NULL, NULL);
   g_return_val_if_fail (spec != NULL, NULL);
 
-  result = (PangoXFont *)g_type_create_instance (PANGO_TYPE_X_FONT);
+  result = g_object_new (PANGO_TYPE_X_FONT, NULL);
   
   result->fontmap = fontmap;
-  g_object_ref (G_OBJECT (fontmap));
+  g_object_ref (fontmap);
   result->display = pango_x_fontmap_get_display (fontmap);
 
   result->fonts = g_strsplit(spec, ",", -1);
@@ -455,6 +334,19 @@ pango_x_load_font (Display    *display,
 
   return (PangoFont *)result;
 }
+
+
+#define FLUSH						\
+  G_STMT_START {					\
+    if (charcount)					\
+      {							\
+	XDrawString16 (display, d, gc,			\
+		       glyph_x0, glyph_y0,		\
+		       xcharbuffer, charcount);		\
+	charcount = 0;					\
+      }							\
+  } G_STMT_END
+
  
 /**
  * pango_x_render:
@@ -477,35 +369,61 @@ pango_x_render  (Display           *display,
 		 int                x, 
 		 int                y)
 {
-  /* Slow initial implementation. For speed, it should really
-   * collect the characters into runs, and draw multiple
-   * characters with each XDrawString16 call.
-   */
   Font old_fid = None;
   XFontStruct *fs;
   int i;
   int x_off = 0;
 
+  /*
+   * We collect the characters in this buffer as long as the font does not
+   * change.  At that time, or when the buffer runs full, or at the end,
+   * then we empty the buffer.
+   */
+  XChar2b xcharbuffer[1000];
+  int glyph_x0 = 0, expected_x = 0; /* x/y initializations are to quiet GCC */
+  int glyph_y0 = 0;
+  int charcount = 0;
+
   g_return_if_fail (display != NULL);
   g_return_if_fail (glyphs != NULL);
   
   for (i=0; i<glyphs->num_glyphs; i++)
     {
-      if (glyphs->glyphs[i].glyph &
-          PANGO_X_UNKNOWN_FLAG)
+      PangoGlyph glyph = glyphs->glyphs[i].glyph;
+      int glyph_x = x + PANGO_PIXELS (x_off + glyphs->glyphs[i].geometry.x_offset);
+      int glyph_y = y + PANGO_PIXELS (glyphs->glyphs[i].geometry.y_offset);
+
+      /* Clip glyphs into the X coordinate range; we really
+       * want to clip glyphs with an ink rect outside the
+       * [0,32767] x [0,32767] rectangle but looking up
+       * the ink rect here would be a noticeable speed hit.
+       * This is close enough.
+       */
+      if (!(glyph &&
+	    glyph_x >= -16384 && glyph_x <= 32767 &&
+	    glyph_y >= -16384 && glyph_y <= 32767))
+	goto next_glyph;
+	      
+      if (glyph & PANGO_X_UNKNOWN_FLAG)
         {
-          PangoFontMetrics *metrics = pango_font_get_metrics (font, NULL);
+	  PangoFontMetrics *metrics = pango_font_get_metrics (font,
+							      pango_language_from_string ("en"));
           int x1, y1, x2, y2; /* rectangle the character should go inside. */
           int baseline;
+	  int stroke_thick;
+
           gunichar wc;
+
+	  FLUSH;
+
+          x1 = glyph_x;
+          y1 = glyph_y - PANGO_PIXELS (metrics->ascent);
+          x2 = x1 + PANGO_PIXELS (glyphs->glyphs[i].geometry.width);
+          y2 = y1 + PANGO_PIXELS (metrics->ascent + metrics->descent);
+          baseline = glyph_y;
+	  stroke_thick = MAX ((int) (0.5 + 0.075 * (y2 - y1)), 1);
           
-          x1 = x + (x_off + glyphs->glyphs[i].geometry.x_offset) / PANGO_SCALE;
-          y1 = y + (glyphs->glyphs[i].geometry.y_offset - metrics->ascent) / PANGO_SCALE;
-          x2 = x1 + glyphs->glyphs[i].geometry.width / PANGO_SCALE;
-          y2 = y1 + (metrics->ascent + metrics->descent) / PANGO_SCALE;
-          baseline = y1 + metrics->ascent / PANGO_SCALE;
-          
-          wc = glyphs->glyphs[i].glyph & (~PANGO_X_UNKNOWN_FLAG);
+          wc = glyph & (~PANGO_X_UNKNOWN_FLAG);
 
           switch (wc)
             {
@@ -518,7 +436,6 @@ pango_x_render  (Display           *display,
                 PangoRectangle up_stroke;
                 PangoRectangle across_stroke;
 
-		int stroke_thick = MAX ((y2 - y1) * 0.075, 1);
 		int hborder = (x2 - x1) * 0.1;
 		int arrow_height = 0.25 * (y2 - y1);
 		int top_border = (y2 - y1) * 0.25;
@@ -564,53 +481,90 @@ pango_x_render  (Display           *display,
               break;
 
             default:
+	      {
+		/* Perhaps we should draw the box-with-numbers as in the
+		 * Xft backend, though we have no guarantee of having
+		 * an appropriate size of font. Right now, we just
+		 * draw an empty box. (To draw the box-with-numbers.
+		 * the backends would have to be changed to use
+		 * pango_x_font_get_unknown_glyph() rather than
+		 * pango_x_get_unknown_glyph().
+		 */
 
-              /* Here we should draw the box-with-numbers as in the
-               * Xft backend. The shaper never gives us a glyph that
-               * activates this case at the moment though, so also
-               * needs hacking.
-               */
-              
-              break;
-            }
+		int xspace = MAX ((int) (0.5 + 0.1 * (x2 - x1)), 1);
+		int yspace = MAX ((int) (0.5 + 0.1 * (y2 - y1)), 1);
+
+		x1 += xspace;
+		x2 -= xspace;
+		y1 += yspace;
+		y2 -= yspace;
+
+		XFillRectangle (display, d, gc,
+				x1, y1,
+				x2 - x1, stroke_thick);
+		XFillRectangle (display, d, gc,
+				x1, y1 + stroke_thick,
+				stroke_thick, y2 - y1 - 2 * stroke_thick);
+		XFillRectangle (display, d, gc,
+				x2 - stroke_thick, y1 + stroke_thick,
+				stroke_thick, y2 - y1 - 2 * stroke_thick);
+		XFillRectangle (display, d, gc,
+				x1, y2 - stroke_thick,
+				x2 - x1, stroke_thick);
+		
+		break;
+	      }
+	    }
 
 	  pango_font_metrics_unref (metrics);
         }
-      else if (glyphs->glyphs[i].glyph)
+      else
 	{
-	  guint16 index = PANGO_X_GLYPH_INDEX (glyphs->glyphs[i].glyph);
-	  guint16 subfont_index = PANGO_X_GLYPH_SUBFONT (glyphs->glyphs[i].glyph);
+	  guint16 index = PANGO_X_GLYPH_INDEX (glyph);
+	  guint16 subfont_index = PANGO_X_GLYPH_SUBFONT (glyph);
 	  PangoXSubfontInfo *subfont;
       
-	  XChar2b c;
-	  
 	  subfont = pango_x_find_subfont (font, subfont_index);
 	  if (subfont)
 	    {
-	      c.byte1 = index / 256;
-	      c.byte2 = index % 256;
-	      
 	      fs = pango_x_get_font_struct (font, subfont);
 	      if (!fs)
 		continue;
 	      
 	      if (fs->fid != old_fid)
 		{
+		  FLUSH;
 		  XSetFont (display, gc, fs->fid);
 		  old_fid = fs->fid;
 		}
-	      
-	      XDrawString16 (display, d, gc,
-			     x + (x_off + glyphs->glyphs[i].geometry.x_offset) / PANGO_SCALE,
-			     y + glyphs->glyphs[i].geometry.y_offset / PANGO_SCALE,
-			     &c, 1);
+
+	      if (charcount == G_N_ELEMENTS (xcharbuffer) ||
+		  (charcount > 0 && (glyph_y != glyph_y0 ||
+				     glyph_x != expected_x)))
+		FLUSH;
+
+	      if (charcount == 0)
+		{
+		  glyph_x0 = glyph_x;
+		  glyph_y0 = glyph_y;
+		}
+	      xcharbuffer[charcount].byte1 = index / 256;
+	      xcharbuffer[charcount].byte2 = index % 256;
+
+	      expected_x = glyph_x + XTextWidth16 (fs, &xcharbuffer[charcount], 1);
+
+	      charcount++;
 	    }
 	}
 
+    next_glyph:
       x_off += glyphs->glyphs[i].geometry.width;
     }
+  FLUSH;
 }
 
+#undef FLUSH
+
 static void
 pango_x_font_get_glyph_extents  (PangoFont      *font,
 				 PangoGlyph      glyph,
@@ -622,7 +576,11 @@ pango_x_font_get_glyph_extents  (PangoFont      *font,
 
   if (glyph & PANGO_X_UNKNOWN_FLAG)
     {
+      PangoFontMetrics *metrics = pango_font_get_metrics (font,
+							  pango_language_from_string ("en"));
       gunichar wc;
+      gdouble width_factor;
+      int w;
       
       wc = glyph & (~PANGO_X_UNKNOWN_FLAG);
           
@@ -633,37 +591,37 @@ pango_x_font_get_glyph_extents  (PangoFont      *font,
         case 0x2028: /* Line separator */
         case 0x2029: /* Paragraph separator */
           {
-            /* Size of carriage-return thingy */
-            PangoFontMetrics *metrics = pango_font_get_metrics (font, NULL);
-            int w;
+#define MAGIC_FACTOR 1.2
 
-#define MAGIC_FACTOR 1.75
+            /* carriage-return thingy */
+	    width_factor = MAGIC_FACTOR;
+	    break;
+	  }
+	default:
+	  {
+	    /* Unknown glyph square */
+	    width_factor = 0.7;
+	  }
+	}
+	    
+      w = metrics->approximate_char_width * width_factor;
+      w = PANGO_SCALE * PANGO_PIXELS (w);
             
-            w = metrics->approximate_char_width * MAGIC_FACTOR;
-            
-            if (ink_rect)
-              {
-                ink_rect->x = 0;
-                ink_rect->width = w;
-                ink_rect->y = - metrics->ascent;
-                ink_rect->height = metrics->ascent + metrics->descent;
-              }
-            if (logical_rect)
-              {
-                logical_rect->x = 0;
-                logical_rect->width = w;
-                logical_rect->y = - metrics->ascent;
-                logical_rect->height = metrics->ascent + metrics->descent;
-              }
-
-	    pango_font_metrics_unref (metrics);
-          }
-          break;
-          
-        default:
-              
-          break;
-        }
+      if (ink_rect)
+	{
+	  ink_rect->x = 0;
+	  ink_rect->width = w;
+	  ink_rect->y = - metrics->ascent;
+	  ink_rect->height = metrics->ascent + metrics->descent;
+	}
+      if (logical_rect)
+	{
+	  logical_rect->x = 0;
+	  logical_rect->width = w;
+	  logical_rect->y = - metrics->ascent;
+	  logical_rect->height = metrics->ascent + metrics->descent;
+	}
+      
     }
   else if (glyph && pango_x_find_glyph (font, glyph, &subfont, &cs))
     {
@@ -898,10 +856,14 @@ get_subfonts_foreach (PangoFont      *font,
 		      gpointer        data)
 {
   GSList **subfonts = data;
+  PangoGlyph glyph = glyph_info->glyph;
 
-  PangoXSubfont subfont = PANGO_X_GLYPH_SUBFONT (glyph_info->glyph);
-  if (!g_slist_find (*subfonts, GUINT_TO_POINTER ((guint)subfont)))
-    *subfonts = g_slist_prepend (*subfonts, GUINT_TO_POINTER ((guint)subfont));
+  if ((glyph & PANGO_X_UNKNOWN_FLAG) == 0)
+    {
+      PangoXSubfont subfont = PANGO_X_GLYPH_SUBFONT (glyph);
+      if (!g_slist_find (*subfonts, GUINT_TO_POINTER ((guint)subfont)))
+	*subfonts = g_slist_prepend (*subfonts, GUINT_TO_POINTER ((guint)subfont));
+    }
 }
 
 /* Get composite font metrics for all subfonts resulting from shaping
@@ -1059,9 +1021,6 @@ pango_x_insert_subfont (PangoFont *font, const char *xlfd)
   
   info->xlfd = g_strdup (xlfd);
   info->font_struct = NULL;
-  info->n_ligs = 0;
-  info->ligs = 0;
-  info->ligature_sets = 0;
 
   xfont->n_subfonts++;
   
@@ -1264,13 +1223,6 @@ subfonts_foreach (gpointer key, gpointer value, gpointer data)
   g_free (value);
 }
 
-static void
-free_sets_foreach (gpointer key, gpointer value, gpointer data)
-{
-  g_free (key);
-  pango_int_set_destroy (value);
-}
-
 static void
 free_metrics_info (PangoXMetricsInfo *info)
 {
@@ -1284,7 +1236,7 @@ pango_x_font_finalize (GObject *object)
   PangoXFont *xfont = (PangoXFont *)object;
   PangoXFontCache *cache = pango_x_font_map_get_font_cache (xfont->fontmap);
 
-  int i, j;
+  int i;
 
   for (i=0; i<xfont->n_subfonts; i++)
     {
@@ -1295,20 +1247,6 @@ pango_x_font_finalize (GObject *object)
       if (info->font_struct)
 	pango_x_font_cache_unload (cache, info->font_struct);
 
-      if (info->ligs) 
-	{
-	  
-	  for (j=0; j<info->n_ligs;j++) 
-	    {
-	      g_free (info->ligs[j].source);
-	    }
-	  
-	  g_free (info->ligs);
-	  
-	  g_hash_table_foreach (info->ligature_sets, free_sets_foreach, NULL);
-	  g_hash_table_destroy (info->ligature_sets);
-	}
-
       g_free (info);
     }
 
@@ -1323,7 +1261,7 @@ pango_x_font_finalize (GObject *object)
   if (xfont->xface)
     pango_x_face_remove (xfont->xface, (PangoFont *)xfont);
 
-  g_object_unref (G_OBJECT (xfont->fontmap));
+  g_object_unref (xfont->fontmap);
 
   g_strfreev (xfont->fonts);
 
@@ -1461,42 +1399,18 @@ pango_x_find_glyph (PangoFont *font,
  * pango_x_get_unknown_glyph:
  * @font: a #PangoFont.
  * 
- * Returns the index of a glyph suitable for drawing unknown characters.
+ * Returns the index of a glyph suitable for drawing unknown characters;
+ * you should generally use pango_x_font_get_unknown_glyph_instead,
+ * since that may return a glyph that provides a better representation
+ * of a particular char. (E.g., by showing hex digits, or a glyph
+ * representive of a certain Unicode range.)
  * 
  * Return value: a glyph index into @font.
  **/
 PangoGlyph
 pango_x_get_unknown_glyph (PangoFont *font)
 {
-  PangoXFont *xfont = (PangoXFont *)font;
-  
-  /* The strategy here is to find _a_ X font, any X font in the fontset, and
-   * then get the unknown glyph for that font.
-   */
-
-  g_return_val_if_fail (font != 0, 0);
-  g_return_val_if_fail (xfont->n_fonts != 0, 0);
-
-  if (xfont->n_subfonts == 0)
-    {
-      int count;
-      char **names = XListFonts (xfont->display, xfont->fonts[0], 1, &count);
-
-      if (count > 0)
-	pango_x_insert_subfont (font, names[0]);
-
-      XFreeFontNames (names);
-    }
-
-  if (xfont->n_subfonts > 0)
-    {
-      XFontStruct *font_struct = pango_x_get_font_struct (font, xfont->subfonts[0]);
-
-      if (font_struct)
-	return PANGO_X_MAKE_GLYPH (1,font_struct->default_char);
-    }
-
-  return 0;
+  return PANGO_X_UNKNOWN_FLAG;
 }
 
 /**
@@ -1619,72 +1533,32 @@ pango_x_render_layout (Display         *display,
 		       int              x, 
 		       int              y)
 {
-  PangoRectangle logical_rect;
-  GSList *tmp_list;
-  PangoAlignment align;
-  int indent;
-  int width;
-  int y_offset = 0;
+  PangoLayoutIter *iter;
 
-  gboolean first = TRUE;
-  
   g_return_if_fail (display != NULL);
-  g_return_if_fail (layout != NULL);
+  g_return_if_fail (PANGO_IS_LAYOUT (layout));
 
-  indent = pango_layout_get_indent (layout);
-  width = pango_layout_get_width (layout);
-  align = pango_layout_get_alignment (layout);
+  iter = pango_layout_get_iter (layout);
 
-  if (width == -1 && align != PANGO_ALIGN_LEFT)
+  do
     {
-      pango_layout_get_extents (layout, NULL, &logical_rect);
-      width = logical_rect.width;
-    }
-  
-  tmp_list = pango_layout_get_lines (layout);
-  while (tmp_list)
-    {
-      PangoLayoutLine *line = tmp_list->data;
-      int x_offset;
+      PangoRectangle   logical_rect;
+      PangoLayoutLine *line;
+      int              baseline;
+      
+      line = pango_layout_iter_get_line (iter);
+      
+      pango_layout_iter_get_line_extents (iter, NULL, &logical_rect);
+      baseline = pango_layout_iter_get_baseline (iter);
       
-      pango_layout_line_get_extents (line, NULL, &logical_rect);
-
-      if (width != 1 && align == PANGO_ALIGN_RIGHT)
-	x_offset = width - logical_rect.width;
-      else if (width != 1 && align == PANGO_ALIGN_CENTER)
-	x_offset = (width - logical_rect.width) / 2;
-      else
-	x_offset = 0;
-
-      if (first)
-	{
-	  if (indent > 0)
-	    {
-	      if (align == PANGO_ALIGN_LEFT)
-		x_offset += indent;
-	      else
-		x_offset -= indent;
-	    }
-
-	  first = FALSE;
-	}
-      else
-	{
-	  if (indent < 0)
-	    {
-	      if (align == PANGO_ALIGN_LEFT)
-		x_offset -= indent;
-	      else
-		x_offset += indent;
-	    }
-	}
-	  
       pango_x_render_layout_line (display, drawable, gc,
-				  line, x + x_offset / PANGO_SCALE, y + (y_offset - logical_rect.y) / PANGO_SCALE);
-
-      y_offset += logical_rect.height;
-      tmp_list = tmp_list->next;
+                                  line,
+                                  x + PANGO_PIXELS (logical_rect.x),
+                                  y + PANGO_PIXELS (baseline));
     }
+  while (pango_layout_iter_next_line (iter));
+
+  pango_layout_iter_free (iter);
 }
 
 /* This utility function is duplicated here and in pango-layout.c; should it be
@@ -1740,243 +1614,17 @@ pango_x_get_item_properties (PangoItem      *item,
     }
 }
 
-static void 
-font_struct_get_ligatures (PangoFontMap *fontmap,
-                           Display *display, 
-                           XFontStruct *fs,
-                           PangoXSubfontInfo *info)
-{
-  int i;
-   
-  PangoXLigatureInfo *linfo = 0;
-  int n_linfo = 0;
-
-  GList *list = g_list_append (NULL, g_strdup ("PANGO_LIGATURE_HACK"));
-  GList *list_start = list;
-
-  info->ligature_sets = g_hash_table_new (g_str_hash, g_str_equal);
-   
-  while (list) 
-    {
-      Atom this_atom = pango_x_fontmap_atom_from_name (fontmap, (char *)list->data);
-      for (i = 0; i < fs->n_properties; i++)
-        {
-          if (fs->properties[i].name == this_atom) 
-            {
-              char *val = g_strdup (pango_x_fontmap_name_from_atom (fontmap, fs->properties[i].card32));
-              char *p;
-              char *a = my_strtok_r (val, " ", &p);
-              while (a)
-                {
-                  char *r;
-                  char *m;
-                  char *q;
-                  PangoXLigatureSource *source = NULL;
-                  gunichar *dest = NULL;
-                  int n_source = 0;
-                  int n_dest = 0;
-                  PangoXLigatureInfo *xli;
-                  
-                  switch (*a) 
-                    {
-                      
-                    case '$': 
-                      /* class being defined */
-                      {
-                        char *name = a + 1;
-                        char *data = strchr (a, '=');
-                        PangoIntSet *set;
-                        if (!data) 
-                          {
-#ifdef PANGO_LIGATURE_HACK_DEBUG
-                            g_warning ("Error parsing ligature info: Isolated $.\n");
-#endif 
-                            break;
-                          }
-                        
-                        *data = 0;
-                        data++;
-                        set = parse_gintset_spec (data);
-                        if (!set) 
-                          {
-#ifdef PANGO_LIGATURE_HACK_DEBUG
-                            g_warning ("Error parsing ligature info: Invalid glyphset.\n");
-#endif
-                            break;
-                          }
-                        g_hash_table_insert (info->ligature_sets, 
-                                             g_strdup (name), set);
-                        break;
-                      }
-                      
-                    case ':': 
-                      /* a pointer */
-                      {
-                        char *lang = a+1;
-                        char *name = strchr (lang, ':');
-                        if (name) 
-                          {
-                            name++;
-                            list = g_list_append (list, g_strdup (name));
-                          }
-                        else 
-			  {
-#ifdef PANGO_LIGATURE_HACK_DEBUG
-			    g_warning ("Error parsing ligature info: Bad pointer.\n");
-#endif
-			  }
-                        break;
-                      }
-                      
-                    default:
-                      /* a literal */
-                      {
-                        n_linfo++;
-                        linfo = g_realloc (linfo, sizeof (PangoXLigatureInfo) * 
-                                           n_linfo);
-                        r = strchr (a, '=');
-                        if (!r) 
-                          {
-#ifdef PANGO_LIGATURE_HACK_DEBUG
-                            g_warning ("Error parsing ligature info: No equals.\n");
-#endif
-                            n_linfo--;
-                            break;
-                          }
-                        *r = 0;
-                        r++;
-                        q = a;
-                        q = my_strtok_r (q, "+", &m);
-                        while (q) 
-                          {
-                            n_source ++;
-                            source = g_realloc (source, n_source * 
-                                                sizeof (PangoXLigatureSource));
-                            if (q[0] == '%') 
-                              {
-                                source[n_source-1].is_set = 1;
-                                source[n_source-1].data.set = 
-                                  g_hash_table_lookup (info->ligature_sets, 
-                                                       q+1);
-                                if (!source[n_source-1].data.set) 
-				  {
-#ifdef PANGO_LIGATURE_HACK_DEBUG
-				    g_warning ("Error parsing ligature info: Unable to locate glyphset : %s\n", q+1);
-#endif
-				    source [n_source-1].is_set = 0;
-				    source [n_source-1].data.glyph = 0;
-				  }
-                              } 
-                            else 
-                              {
-                                int i = hex_to_integer (q);
-                                if (i == -1) 
-				  {
-#ifdef PANGO_LIGATURE_HACK_DEBUG
-				    g_warning ("Error parsing ligature info: Bad character value : %s. Assuming 0\n", q);
-#endif
-				    i = 0;
-				  }
-                                source [n_source-1].is_set = 0;
-                                source [n_source-1].data.glyph = i;
-                              }
-                            q = my_strtok_r (NULL, "+", &m);
-                          }
-                        q = r;
-                        q = my_strtok_r (q, "+", &m);
-                        while (q) 
-                          {
-                            n_dest++;
-                            dest = g_realloc (dest, n_dest * sizeof (gunichar));
-                            
-                            if (q[0] == '%') 
-			      {
-				char *er;
-				dest[n_dest-1] = -strtol (q+1, &er, 10);
-				if (*er) 
-				  {
-#ifdef PANGO_LIGATURE_HACK_DEBUG
-				    g_warning ("Error parsing ligature info: Bad %% reference. Assuming 1");
-#endif
-				    dest[n_dest-1] = -1;
-				  }
-			      }
-                            else 
-			      {
-				int i = hex_to_integer (q);
-				if (i != -1) 
-				  {
-				    dest[n_dest-1] = i;
-				  } 
-				else 
-				  {
-				    dest[n_dest-1] = 0;
-				  }
-			      }
-                            
-                            q = my_strtok_r (NULL, "+", &m);
-                          }
-                        
-                        xli = linfo + n_linfo - 1;
-                        
-                        xli->source = source;
-                        xli->n_source = n_source;
-                        xli->dest = dest;
-                        xli->n_dest = n_dest;
-                        
-                        if (xli->n_dest > xli->n_source)
-                          {
-                            g_warning ("Error parsing ligature info: Warning : truncating substitute string.");
-                            xli->n_dest = n_source;
-                          }
-                      }
-                    }
-                  
-                  /* end switch */
-                  a = my_strtok_r (NULL, " ", &p);
-                }
-              g_free (val);
-            }
-        }
-      list = g_list_next (list);
-    }
-  
-  list = list_start;
-  
-  while (list) 
-    {
-      g_free (list->data);
-      list = g_list_next (list);
-    }
-
-  g_list_free (list_start);
-  
-  info->n_ligs = n_linfo;
-  info->ligs = linfo;
-}
-
 /**
  * pango_x_apply_ligatures:
- * @font: a #PangoFont.
- * @subfont: a #PangoXSubFont.
- * @glyphs: a pointer to a pointer to an array of
- *          glyph indices. This holds the input glyphs
- *          on entry, and ligation will be performed
- *          on this array in-place. If the number
- *          of glyphs is increased, Pango will
- *          call g_realloc() on @glyphs, so @chars
- *          must be allocated with g_malloc().
- * @n_glyphs: a pointer to the number of glyphs
- *            *@n_glyphs is the number of original glyphs
- *            on entry and the number of resulting glyphs
- *            upon return.
- * @clusters: a pointer to the cluster information.
+ * @font: unused
+ * @subfont: unused
+ * @glyphs: unused
+ * @n_glyphs: unused
+ * @clusters: unused
  *
- * Does subfont-specific ligation.  This involves replacing
- * groups of glyphs in @chars with alternate groups of glyphs
- * based on information provided in the X font.
+ * Previously did subfont-specific ligation. Now a no-op.
  *
- * Return value: %TRUE if any ligations were performed.
+ * Return value: %FALSE, always.
  */
 gboolean
 pango_x_apply_ligatures (PangoFont     *font, 
@@ -1985,81 +1633,7 @@ pango_x_apply_ligatures (PangoFont     *font,
                          int           *n_glyphs,
                          int           **clusters) 
 {
-  int hits = 0;
-  int i, j, k;
-  PangoXSubfontInfo *subfont;
-  PangoXLigatureInfo *linfo;
-  int n_linfo = 0;
-  XFontStruct *fs;
-  
-  g_return_val_if_fail (font != NULL, 0);
-
-  subfont = pango_x_find_subfont (font, subfont_id);
-  if (!subfont)
-    return 0;
-
-  fs = pango_x_get_font_struct (font, subfont);
-  if (!fs)
-    return 0;
-
-  linfo = subfont->ligs;
-  n_linfo = subfont->n_ligs;
-  
-  for (i = 0; i < *n_glyphs; i++) 
-    for (j= 0; j < n_linfo; j++) 
-      {
-        PangoXLigatureInfo *li = &linfo[j];
-        gunichar *temp;
-        
-        if (i + li->n_source > *n_glyphs) 
-          continue;
-        
-        for (k = 0; k < li->n_source; k++) 
-          {
-            if ((li->source[k].is_set && 
-                 !pango_int_set_contains (li->source[k].data.set,
-                                          (*glyphs)[i + k]))
-                || (!li->source[k].is_set && 
-                    (*glyphs)[i + k] != li->source[k].data.glyph))
-              goto next_pattern;
-          }
-        
-
-        {
-          gunichar buffer[16];
-          if (li->n_source < G_N_ELEMENTS (buffer))
-            {
-              memcpy (buffer, &(*glyphs)[i], li->n_source * sizeof (gunichar));
-              temp = buffer;
-            }
-          else
-            { 
-              temp = g_memdup (&(*glyphs)[i], li->n_source * sizeof (gunichar));
-            }
-          
-          for (k = 0; k < li->n_dest; k++) 
-            {
-              int f = li->dest[k];
-              if (f < 0) 
-                f = temp[i - (1+f)];
-              
-              (*glyphs) [i + k - (li->n_dest - li->n_source)] = f;
-            }
-          
-          for (k = 0; k < li->n_source-li->n_dest; k++)
-            (*glyphs) [i+k] = 0;
-          
-          hits++;
-          i += li->n_source - 1;
-
-          if (temp != buffer) 
-            g_free (temp);
-        }
-
-      next_pattern: ;
-      }
-  
-  return hits >= 1;
+  return FALSE;
 }
   
 /**
@@ -2161,16 +1735,5 @@ pango_x_font_get_unknown_glyph (PangoFont *font,
 {
   g_return_val_if_fail (PANGO_IS_FONT (font), 0);
 
-  switch (wc)
-    {
-    case '\n':
-    case '\r':
-    case 0x2028: /* Line separator */
-    case 0x2029: /* Paragraph separator */
-      return PANGO_X_UNKNOWN_FLAG | wc;
-      break;
-    default:
-      return 0;
-      break;
-    }
+  return PANGO_X_UNKNOWN_FLAG | wc;
 }
